@@ -21,8 +21,17 @@
 #ifndef __BTRFS_CHECK_MODE_COMMON_H__
 #define __BTRFS_CHECK_MODE_COMMON_H__
 
+#include "kerncompat.h"
 #include <sys/stat.h>
+#include <stdbool.h>
+#include "kernel-lib/list.h"
 #include "kernel-shared/ctree.h"
+#include "common/messages.h"
+
+struct btrfs_trans_handle;
+struct extent_buffer;
+
+extern struct task_ctx g_task_ctx;
 
 #define FREE_SPACE_CACHE_INODE_MODE	(0100600)
 /*
@@ -73,7 +82,6 @@ extern int no_holes;
 extern int init_extent_tree;
 extern int check_data_csum;
 extern struct btrfs_fs_info *gfs_info;
-extern struct task_ctx ctx;
 extern struct cache_tree *roots_info_cache;
 
 static inline u8 imode_to_type(u32 imode)
@@ -93,11 +101,11 @@ static inline u8 imode_to_type(u32 imode)
 #undef S_SHIFT
 }
 
-static inline int fs_root_objectid(u64 objectid)
+static inline bool fs_root_objectid(u64 objectid)
 {
 	if (objectid == BTRFS_TREE_RELOC_OBJECTID ||
 	    objectid == BTRFS_DATA_RELOC_TREE_OBJECTID)
-		return 1;
+		return true;
 	return is_fstree(objectid);
 }
 
