@@ -38,6 +38,7 @@
 #include "common/units.h"
 #include "common/help.h"
 #include "common/device-utils.h"
+#include "common/sysfs-utils.h"
 #include "common/messages.h"
 #include "common/path-utils.h"
 #include "cmds/filesystem-usage.h"
@@ -144,7 +145,7 @@ static int load_chunk_info(int fd, struct chunk_info **chunkinfo_ret,
 	struct btrfs_ioctl_search_key *sk = &args.key;
 	struct btrfs_ioctl_search_header *sh;
 	unsigned long off = 0;
-	int i, e;
+	int i;
 
 	memset(&args, 0, sizeof(args));
 
@@ -167,11 +168,9 @@ static int load_chunk_info(int fd, struct chunk_info **chunkinfo_ret,
 
 	while (1) {
 		ret = ioctl(fd, BTRFS_IOC_TREE_SEARCH, &args);
-		e = errno;
-		if (e == EPERM)
-			return -e;
-
 		if (ret < 0) {
+			if (errno == EPERM)
+				return -errno;
 			error("cannot look up chunk tree info: %m");
 			return 1;
 		}
@@ -775,7 +774,7 @@ static int load_device_info(int fd, struct device_info **devinfo_ret,
 			goto out;
 		}
 		memset(&dev_info, 0, sizeof(dev_info));
-		ret = get_device_info(fd, i, &dev_info);
+		ret = device_get_info(fd, i, &dev_info);
 
 		if (ret == -ENODEV)
 			continue;
