@@ -23,11 +23,11 @@
 #include <stdio.h>
 #include <string.h>
 #include <stdbool.h>
+#include "kernel-shared/uapi/btrfs.h"
 #include "common/help.h"
 #include "common/open-utils.h"
 #include "common/messages.h"
 #include "cmds/commands.h"
-#include "ioctl.h"
 
 static const char * const quota_cmd_group_usage[] = {
 	"btrfs quota <command> [options] <path>",
