@@ -24,12 +24,12 @@
 #include <stdbool.h>
 #include <stddef.h>
 #include "kernel-lib/sizes.h"
+#include "kernel-shared/uapi/btrfs.h"
 #include "kernel-shared/ctree.h"
 #include "common/defs.h"
 #include "common/internal.h"
 #include "common/messages.h"
 #include "common/fsfeatures.h"
-#include "ioctl.h"
 
 enum exclusive_operation {
 	BTRFS_EXCLOP_NONE,
@@ -59,8 +59,6 @@ int ask_user(const char *question);
 int lookup_path_rootid(int fd, u64 *rootid);
 int find_mount_fsroot(const char *subvol, const char *subvolid, char **mount);
 int find_mount_root(const char *path, char **mount_root);
-int get_device_info(int fd, u64 devid,
-		struct btrfs_ioctl_dev_info_args *di_args);
 int get_df(int fd, struct btrfs_ioctl_space_args **sargs_ret);
 
 const char *subvol_strip_mountpoint(const char *mnt, const char *full_path);
@@ -112,11 +110,6 @@ char *btrfs_test_for_multiple_profiles(int fd);
 int btrfs_warn_multiple_profiles(int fd);
 void btrfs_warn_experimental(const char *str);
 
-int sysfs_open_file(const char *name);
-int sysfs_open_fsid_file(int fd, const char *filename);
-int sysfs_read_file(int fd, char *buf, size_t size);
-int sysfs_open_fsid_dir(int fd, const char *dirname);
-
 /* An error code to error string mapping for the kernel error codes */
 static inline char *btrfs_err_str(enum btrfs_err_code err_code)
 {
