@@ -23,7 +23,14 @@
 #define __BTRFS_MKFS_COMMON_H__
 
 #include "kerncompat.h"
+#include <stdbool.h>
+#include "kernel-lib/sizes.h"
+#include "kernel-shared/ctree.h"
 #include "common/defs.h"
+#include "common/fsfeatures.h"
+
+struct btrfs_root;
+struct btrfs_trans_handle;
 
 #define BTRFS_MKFS_SYSTEM_GROUP_SIZE SZ_4M
 #define BTRFS_MKFS_SMALL_VOLUME_SIZE SZ_1G
@@ -37,9 +44,6 @@
 #define BTRFS_MKFS_DEFAULT_DATA_MULTI_DEVICE	0	/* SINGLE */
 #define BTRFS_MKFS_DEFAULT_META_MULTI_DEVICE	BTRFS_BLOCK_GROUP_RAID1
 
-struct btrfs_trans_handle;
-struct btrfs_root;
-
 /*
  * Tree root blocks created during mkfs
  */
@@ -75,10 +79,7 @@ struct btrfs_mkfs_config {
 	u32 sectorsize;
 	u32 stripesize;
 	u32 leaf_data_size;
-	/* Bitfield of incompat features, BTRFS_FEATURE_INCOMPAT_* */
-	u64 features;
-	/* Bitfield of BTRFS_RUNTIME_FEATURE_* */
-	u64 runtime_features;
+	struct btrfs_mkfs_features features;
 	/* Size of the filesystem in bytes */
 	u64 num_bytes;
 	/* checksum algorithm to use */
@@ -105,7 +106,7 @@ int test_minimum_size(const char *file, u64 min_dev_size);
 int is_vol_small(const char *file);
 int test_num_disk_vs_raid(u64 metadata_profile, u64 data_profile,
 	u64 dev_cnt, int mixed, int ssd);
-int test_status_for_mkfs(const char *file, bool force_overwrite);
-int test_dev_for_mkfs(const char *file, int force_overwrite);
+bool test_status_for_mkfs(const char *file, bool force_overwrite);
+bool test_dev_for_mkfs(const char *file, int force_overwrite);
 
 #endif
